@@ -25,6 +25,11 @@
 #include <process.h>
 #endif
 
+#if defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0600
+#include <winreg.h>
+#include <winerror.h>
+#endif
+
 #include <boolean.h>
 #include <compat/strl.h>
 #include <dynamic/dylib.h>
@@ -141,9 +146,9 @@ const struct win32_lang_pair win32_lang_pairs[] =
 
 unsigned short win32_get_langid_from_retro_lang(enum retro_language lang)
 {
-   int i;
+   size_t i;
 
-   for (i = 0; i < sizeof(win32_lang_pairs) / sizeof(win32_lang_pairs[0]); i++)
+   for (i = 0; i < ARRAY_SIZE(win32_lang_pairs); i++)
    {
       if (win32_lang_pairs[i].lang == lang)
          return win32_lang_pairs[i].lang_ident;
@@ -154,9 +159,9 @@ unsigned short win32_get_langid_from_retro_lang(enum retro_language lang)
 
 enum retro_language win32_get_retro_lang_from_langid(unsigned short langid)
 {
-   int i;
+   size_t i;
 
-   for (i = 0; i < sizeof(win32_lang_pairs) / sizeof(win32_lang_pairs[0]); i++)
+   for (i = 0; i < ARRAY_SIZE(win32_lang_pairs); i++)
    {
       if (win32_lang_pairs[i].lang_ident > 0x3ff)
       {
@@ -267,9 +272,10 @@ static void gfx_set_dwm(void)
       g_plat_win32_flags |= PLAT_WIN32_FLAG_DWM_COMPOSITION_DISABLED;
 }
 
-static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
+static size_t frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
 {
-   char buildStr[11]      = {0};
+   size_t _len            = 0;
+   char build_str[11]     = {0};
    bool server            = false;
    const char *arch       = "";
 
@@ -277,15 +283,25 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
    /* Windows 2000 and later */
    SYSTEM_INFO si         = {{0}};
    OSVERSIONINFOEX vi     = {0};
+#ifndef _MSC_VER
+   /* Vista and later, MSYS2/MINGW64 build */
+   const char win_ver_reg_key[]    = "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion";
+   const DWORD reg_read_flags      = RRF_RT_REG_SZ; /* Only read strings (REG_SZ) */
+   const int ProductName_2nd_digit = 9; /* second digit in the string 'Windows 10' */
+   char str_ProductName[64]        = {0};
+   char str_DisplayVersion[64]     = {0};
+   char str_LCUVer[64]             = {0};
+   char str_CurrentBuild[64]       = {0};
+   DWORD key_type                  = 0; /* null pointer */
+   DWORD data_size                 = 0;
+   long reg_read_result;
+   bool read_success               = TRUE;
+   /* end Vista and later; still within Windows 2000 and later block */
+#endif
+
    vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
 
    GetSystemInfo(&si);
-
-   /* Available from NT 3.5 and Win95 */
-   GetVersionEx((OSVERSIONINFO*)&vi);
-
-   server = vi.wProductType != VER_NT_WORKSTATION;
-
    switch (si.wProcessorArchitecture)
    {
       case PROCESSOR_ARCHITECTURE_AMD64:
@@ -300,6 +316,73 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
       default:
          break;
    }
+
+#ifndef _MSC_VER
+   /* Vista and later, MSYS2/MINGW64 build
+    * Check for Win11 by looking for a specific Registry value.
+    * The behavior of GetVersionEx is changed under Win11 and no longer provides
+    * relevant data. If the specific Registry value is present, read version data
+    * directly from registry and skip remainder of function.
+    * Each read is paired for string values; the first gets the size of the
+    * string (read into data_size); the second passes data_size back as an
+    * argument and reads the actual string. */
+   reg_read_result = RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "LCUVer",
+         reg_read_flags, &key_type, 0, &data_size);
+   
+   if (reg_read_result == ERROR_SUCCESS)
+   {
+      if (RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "LCUVer",
+            reg_read_flags, &key_type, str_LCUVer, &data_size) != ERROR_SUCCESS)
+         read_success = FALSE;
+
+      if (RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "ProductName",
+            reg_read_flags, &key_type, 0, &data_size) != ERROR_SUCCESS)
+         read_success = FALSE;
+      
+      if (RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "ProductName",
+            reg_read_flags, &key_type, str_ProductName, &data_size) != ERROR_SUCCESS)
+         read_success = FALSE;
+
+      if (RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "DisplayVersion",
+            reg_read_flags, &key_type, 0, &data_size) != ERROR_SUCCESS)
+         read_success = FALSE;
+
+      if (RegGetValue(HKEY_LOCAL_MACHINE, win_ver_reg_key, "DisplayVersion",
+            reg_read_flags, &key_type, str_DisplayVersion, &data_size) != ERROR_SUCCESS)
+         read_success = FALSE;
+
+      if (read_success)
+      {
+         str_ProductName[ProductName_2nd_digit] = '1';
+         /* Even the version in the Registry still says Windows 10 and requires
+          * string manipulation. */
+         
+          _len = strlcpy(s, str_ProductName, len);
+                  if (!string_is_empty(arch))
+         {
+            _len += strlcat(s, " ",  len);
+            _len += strlcat(s, arch, len);
+         }
+         _len = strlcat(s, " ", len);
+         _len = strlcat(s, str_DisplayVersion, len);
+         _len = strlcat(s, " (", len);
+         _len = strlcat(s, str_LCUVer, len);
+         _len = strlcat(s, ")", len);
+
+         *major = 10;
+         *minor = 0;
+         
+         return _len;
+      }
+   }
+   /* End registry-check-and-read code; still within 2000-and-later block */
+#endif
+
+   /* GetVersionEx call changed in Win2K and later */
+   GetVersionEx((OSVERSIONINFO*)&vi);
+
+   server = vi.wProductType != VER_NT_WORKSTATION;
+
 #else
    OSVERSIONINFO vi = {0};
    vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
@@ -315,46 +398,46 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
       *minor = vi.dwMinorVersion;
 
    if (vi.dwMajorVersion == 4 && vi.dwMinorVersion == 0)
-      snprintf(buildStr, sizeof(buildStr), "%lu", (DWORD)(LOWORD(vi.dwBuildNumber))); /* Windows 95 build number is in the low-order word only */
+      snprintf(build_str, sizeof(build_str), "%lu", (DWORD)(LOWORD(vi.dwBuildNumber))); /* Windows 95 build number is in the low-order word only */
    else
-      snprintf(buildStr, sizeof(buildStr), "%lu", vi.dwBuildNumber);
+      snprintf(build_str, sizeof(build_str), "%lu", vi.dwBuildNumber);
 
    switch (vi.dwMajorVersion)
    {
       case 10:
-         if (atoi(buildStr) >= 21996)
-            strlcpy(s, "Windows 11", len);
+         if (atoi(build_str) >= 21996)
+            _len = strlcpy(s, "Windows 11", len);
          else if (server)
-            strlcpy(s, "Windows Server 2016", len);
+            _len = strlcpy(s, "Windows Server 2016", len);
          else
-            strlcpy(s, "Windows 10", len);
+            _len = strlcpy(s, "Windows 10", len);
          break;
       case 6:
          switch (vi.dwMinorVersion)
          {
             case 3:
                if (server)
-                  strlcpy(s, "Windows Server 2012 R2", len);
+                  _len = strlcpy(s, "Windows Server 2012 R2", len);
                else
-                  strlcpy(s, "Windows 8.1", len);
+                  _len = strlcpy(s, "Windows 8.1", len);
                break;
             case 2:
                if (server)
-                  strlcpy(s, "Windows Server 2012", len);
+                  _len = strlcpy(s, "Windows Server 2012", len);
                else
-                  strlcpy(s, "Windows 8", len);
+                  _len = strlcpy(s, "Windows 8", len);
                break;
             case 1:
                if (server)
-                  strlcpy(s, "Windows Server 2008 R2", len);
+                  _len = strlcpy(s, "Windows Server 2008 R2", len);
                else
-                  strlcpy(s, "Windows 7", len);
+                  _len = strlcpy(s, "Windows 7", len);
                break;
             case 0:
                if (server)
-                  strlcpy(s, "Windows Server 2008", len);
+                  _len = strlcpy(s, "Windows Server 2008", len);
                else
-                  strlcpy(s, "Windows Vista", len);
+                  _len = strlcpy(s, "Windows Vista", len);
                break;
             default:
                break;
@@ -366,22 +449,22 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
             case 2:
                if (server)
                {
-                  strlcpy(s, "Windows Server 2003", len);
+                  _len = strlcpy(s, "Windows Server 2003", len);
                   if (GetSystemMetrics(SM_SERVERR2))
-                     strlcat(s, " R2", len);
+                     _len += strlcpy(s + _len, " R2", len - _len);
                }
                else
                {
                   /* Yes, XP Pro x64 is a higher version number than XP x86 */
                   if (string_is_equal(arch, "x64"))
-                     strlcpy(s, "Windows XP", len);
+                     _len = strlcpy(s, "Windows XP", len);
                }
                break;
             case 1:
-               strlcpy(s, "Windows XP", len);
+               _len = strlcpy(s, "Windows XP", len);
                break;
             case 0:
-               strlcpy(s, "Windows 2000", len);
+               _len = strlcpy(s, "Windows 2000", len);
                break;
          }
          break;
@@ -390,40 +473,40 @@ static void frontend_win32_get_os(char *s, size_t len, int *major, int *minor)
          {
             case 0:
                if (vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
-                  strlcpy(s, "Windows 95", len);
+                  _len = strlcpy(s, "Windows 95", len);
                else if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT)
-                  strlcpy(s, "Windows NT 4.0", len);
+                  _len = strlcpy(s, "Windows NT 4.0", len);
                else
-                  strlcpy(s, "Unknown", len);
+                  _len = strlcpy(s, "Unknown", len);
                break;
             case 90:
-               strlcpy(s, "Windows ME", len);
+               _len = strlcpy(s, "Windows ME", len);
                break;
             case 10:
-               strlcpy(s, "Windows 98", len);
+               _len = strlcpy(s, "Windows 98", len);
                break;
          }
          break;
       default:
-         snprintf(s, len, "Windows %i.%i", *major, *minor);
+         _len = snprintf(s, len, "Windows %i.%i", *major, *minor);
          break;
    }
 
    if (!string_is_empty(arch))
    {
-      strlcat(s, " ", len);
-      strlcat(s, arch, len);
+      _len += strlcpy(s + _len, " ",  len - _len);
+      _len += strlcpy(s + _len, arch, len - _len);
    }
 
-   strlcat(s, " Build ", len);
-   strlcat(s, buildStr, len);
+   _len += strlcpy(s + _len, " Build ", len - _len);
+   _len += strlcpy(s + _len, build_str, len - _len);
 
    if (!string_is_empty(vi.szCSDVersion))
    {
-      strlcat(s, " ", len);
-      strlcat(s, vi.szCSDVersion, len);
+      _len += strlcpy(s + _len, " ", len - _len);
+      strlcpy(s + _len, vi.szCSDVersion, len - _len);
    }
-
+   return _len;
 }
 
 static void frontend_win32_init(void *data)
@@ -453,7 +536,7 @@ static void frontend_win32_init(void *data)
 static void init_nvda(void)
 {
 #ifdef HAVE_DYLIB
-   if (     (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA) 
+   if (     (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_NVDA)
          && !nvda_lib)
    {
       if ((nvda_lib = dylib_load("nvdaControllerClient64.dll")))
@@ -565,24 +648,51 @@ static void frontend_win32_env_get(int *argc, char *argv[],
 {
    const char *tmp_dir = getenv("TMP");
    const char *libretro_directory = getenv("LIBRETRO_DIRECTORY");
+   const char *libretro_assets_directory = getenv("LIBRETRO_ASSETS_DIRECTORY");
+   const char* libretro_autoconfig_directory = getenv("LIBRETRO_AUTOCONFIG_DIRECTORY");
+   const char* libretro_cheats_directory = getenv("LIBRETRO_CHEATS_DIRECTORY");
+   const char* libretro_database_directory = getenv("LIBRETRO_DATABASE_DIRECTORY");
+   const char* libretro_system_directory = getenv("LIBRETRO_SYSTEM_DIRECTORY");
+   const char* libretro_video_filter_directory = getenv("LIBRETRO_VIDEO_FILTER_DIRECTORY");
+   const char* libretro_video_shader_directory = getenv("LIBRETRO_VIDEO_SHADER_DIRECTORY");
    if (!string_is_empty(tmp_dir))
       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CACHE],
          tmp_dir, sizeof(g_defaults.dirs[DEFAULT_DIR_CACHE]));
 
    gfx_set_dwm();
 
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_ASSETS],
-      ":\\assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
+   if (!string_is_empty(libretro_assets_directory))
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_ASSETS], libretro_assets_directory,
+	      sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
+   else
+       fill_pathname_expand_special(
+	   g_defaults.dirs[DEFAULT_DIR_ASSETS],
+	   ":\\assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER],
       ":\\filters\\audio", sizeof(g_defaults.dirs[DEFAULT_DIR_AUDIO_FILTER]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER],
-      ":\\filters\\video", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CHEATS],
-      ":\\cheats", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_DATABASE],
-      ":\\database\\rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
+   if (!string_is_empty(libretro_video_filter_directory))
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER],
+	       libretro_video_filter_directory,
+	       sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER],
+           ":\\filters\\video", sizeof(g_defaults.dirs[DEFAULT_DIR_VIDEO_FILTER]));
+   if (!string_is_empty(libretro_cheats_directory))
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_CHEATS],
+	       libretro_cheats_directory,
+	       sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CHEATS],
+           ":\\cheats", sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
+   if (!string_is_empty(libretro_database_directory))
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_DATABASE],
+	       libretro_database_directory,
+	       sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_DATABASE],
+           ":\\database\\rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_PLAYLIST],
-      ":\\playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
+      ":\\playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_PLAYLIST]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG],
       ":\\config\\record", sizeof(g_defaults.dirs[DEFAULT_DIR_RECORD_CONFIG]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_RECORD_OUTPUT],
@@ -597,18 +707,34 @@ static void frontend_win32_env_get(int *argc, char *argv[],
       ":\\thumbnails", sizeof(g_defaults.dirs[DEFAULT_DIR_THUMBNAILS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_OVERLAY],
       ":\\overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
+   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_OSK_OVERLAY],
+      ":\\overlays\\keyboards", sizeof(g_defaults.dirs[DEFAULT_DIR_OSK_OVERLAY]));
    if (!string_is_empty(libretro_directory))
       strlcpy(g_defaults.dirs[DEFAULT_DIR_CORE], libretro_directory,
             sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
    else
       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE],
             ":\\cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE_INFO],
-      ":\\info", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG],
-      ":\\autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SHADER],
-      ":\\shaders", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+   if (!string_is_empty(libretro_directory))
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_CORE_INFO], libretro_directory,
+            sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE_INFO],
+           ":\\info", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
+   if (!string_is_empty(libretro_autoconfig_directory))
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG],
+	      libretro_autoconfig_directory,
+	      sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG],
+             ":\\autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
+   if (!string_is_empty(libretro_video_filter_directory))
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_SHADER],
+	      libretro_video_shader_directory,
+	      sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SHADER],
+             ":\\shaders", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS],
       ":\\downloads", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SCREENSHOT],
@@ -617,8 +743,13 @@ static void frontend_win32_env_get(int *argc, char *argv[],
       ":\\saves", sizeof(g_defaults.dirs[DEFAULT_DIR_SRAM]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SAVESTATE],
       ":\\states", sizeof(g_defaults.dirs[DEFAULT_DIR_SAVESTATE]));
-   fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SYSTEM],
-      ":\\system", sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
+   if (!string_is_empty(libretro_system_directory))
+       strlcpy(g_defaults.dirs[DEFAULT_DIR_SYSTEM],
+	       libretro_system_directory,
+	       sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
+   else
+       fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_SYSTEM],
+             ":\\system", sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
    fill_pathname_expand_special(g_defaults.dirs[DEFAULT_DIR_LOGS],
       ":\\logs", sizeof(g_defaults.dirs[DEFAULT_DIR_LOGS]));
 
@@ -749,23 +880,18 @@ static void frontend_win32_respawn(char *s, size_t len, char *args)
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    char executable_path[PATH_MAX_LENGTH] = {0};
-   char executable_args[PATH_MAX_LENGTH] = {0};
 
    if (win32_fork_mode != FRONTEND_FORK_RESTART)
       return;
 
-   fill_pathname_application_path(executable_path,
-         sizeof(executable_path));
+   GetModuleFileName(NULL, executable_path, PATH_MAX_LENGTH);
    path_set(RARCH_PATH_CORE, executable_path);
 
-   /* Remove executable path from arguments given to CreateProcess */
-   snprintf(executable_args, sizeof(executable_args), "%s", strstr(args, ".exe") + 4);
-
    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    memset(&pi, 0, sizeof(pi));
 
-   if (!CreateProcess(executable_path, executable_args,
+   if (!CreateProcess(executable_path, GetCommandLine(),
          NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
       RARCH_ERR("Failed to restart RetroArch\n");
 }
@@ -829,7 +955,7 @@ static const char *accessibility_win_language_id(const char* language)
       return "401";
    else if (string_is_equal(language,"hu"))
       return "040e";
-   else if (string_is_equal(language,"zh_tw") || string_is_equal(language,"zh"))
+   else if (string_is_equal(language, "zh_tw") || string_is_equal(language,"zh"))
       return "804";
    else if (string_is_equal(language,"el"))
       return "408";
@@ -851,7 +977,7 @@ static const char *accessibility_win_language_id(const char* language)
       return "412";
    else if (string_is_equal(language,"pl"))
       return "415";
-   else if (string_is_equal(language,"cs")) 
+   else if (string_is_equal(language,"cs"))
       return "405";
    return "";
 }
@@ -894,13 +1020,15 @@ static const char *accessibility_win_language_code(const char* language)
       return "Microsoft Naayf Desktop";
    else if (string_is_equal(language,"hu"))
       return "Microsoft Szabolcs Desktop";
-   else if (string_is_equal(language,"zh_tw") || string_is_equal(language,"zh"))
+   else if (string_is_equal(language, "zh_tw")
+            || string_is_equal(language,"zh-TW")
+            || string_is_equal(language,"zh"))
       return "Microsoft Zhiwei Desktop";
    else if (string_is_equal(language,"el"))
       return "Microsoft Stefanos Desktop";
    else if (string_is_equal(language,"ru"))
       return "Microsoft Pavel Desktop";
-   else if (string_is_equal(language,"nb"))
+   else if (string_is_equal(language,"no") || string_is_equal(language,"nb"))
       return "Microsoft Jon Desktop";
    else if (string_is_equal(language,"da"))
       return "Microsoft Helle Desktop";
@@ -908,7 +1036,7 @@ static const char *accessibility_win_language_code(const char* language)
       return "Microsoft Heidi Desktop";
    else if (string_is_equal(language,"zh_hk"))
       return "Microsoft Danny Desktop";
-   else if (string_is_equal(language,"zh_cn"))
+   else if (string_is_equal(language,"zh_cn") || string_is_equal(language,"zh-CN"))
       return "Microsoft Kangkang Desktop";
    else if (string_is_equal(language,"tr"))
       return "Microsoft Tolga Desktop";
@@ -916,8 +1044,24 @@ static const char *accessibility_win_language_code(const char* language)
       return "Microsoft Heami Desktop";
    else if (string_is_equal(language,"pl"))
       return "Microsoft Adam Desktop";
-   else if (string_is_equal(language,"cs")) 
+   else if (string_is_equal(language,"cs"))
       return "Microsoft Jakub Desktop";
+   else if (string_is_equal(language,"vi"))
+      return "Microsoft An Desktop";
+   else if (string_is_equal(language,"hr"))
+      return "Microsoft Matej Desktop";
+   else if (string_is_equal(language,"bg"))
+      return "Microsoft Ivan Desktop";
+   else if (string_is_equal(language,"ms"))
+      return "Microsoft Rizwan Desktop";
+   else if (string_is_equal(language,"sl"))
+      return "Microsoft Lado Desktop";
+   else if (string_is_equal(language,"ta"))
+      return "Microsoft Valluvar Desktop";
+   else if (string_is_equal(language,"en_gb"))
+      return "Microsoft George Desktop";
+   else if (string_is_equal(language,"ca") || string_is_equal(language,"ca_ES@valencia"))
+      return "Microsoft Herena Desktop";
    return "";
 }
 
@@ -946,12 +1090,12 @@ static bool create_win32_process(char* cmd, const char * input)
       size_t input_len = strlen(input);
       if (!CreatePipe(&rd, &wr, NULL, input_len))
          return false;
-      
+
       SetHandleInformation(rd, HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT);
-      
+
       WriteFile(wr, input, input_len, &dummy, NULL);
       CloseHandle(wr);
-      
+
       si.dwFlags    |= STARTF_USESTDHANDLES;
       si.hStdInput   = rd;
       si.hStdOutput  = GetStdHandle(STD_OUTPUT_HANDLE);
@@ -986,7 +1130,7 @@ static bool is_narrator_running_windows(void)
    {
       long res = nvdaController_testIfRunning_func();
 
-      if (res != 0) 
+      if (res != 0)
       {
          /* The running nvda service wasn't found, so revert
             back to the powershell method
@@ -1037,7 +1181,7 @@ static bool accessibility_speak_windows(int speed,
 #ifdef HAVE_NVDA
    init_nvda();
 #endif
-   
+
    if (g_plat_win32_flags & PLAT_WIN32_FLAG_USE_POWERSHELL)
    {
       const char * template_lang = "powershell.exe -NoProfile -WindowStyle Hidden -Command \"Add-Type -AssemblyName System.Speech; $synth = New-Object System.Speech.Synthesis.SpeechSynthesizer; $synth.SelectVoice(\\\"%s\\\"); $synth.Rate = %s; $synth.Speak($input);\"";
@@ -1059,12 +1203,15 @@ static bool accessibility_speak_windows(int speed,
       wchar_t        *wc = utf8_to_utf16_string_alloc(speak_text);
       long res           = nvdaController_testIfRunning_func();
 
-      if (!wc || res != 0) 
+      if (!wc || res != 0)
       {
          RARCH_ERR("Error communicating with NVDA\n");
+         /* Fallback on powershell immediately and retry */
+         g_plat_win32_flags &= ~PLAT_WIN32_FLAG_USE_NVDA;
+         g_plat_win32_flags |= PLAT_WIN32_FLAG_USE_POWERSHELL;
          if (wc)
             free(wc);
-         return false;
+         return accessibility_speak_windows(speed, speak_text, priority);
       }
 
       nvdaController_cancelSpeech_func();
