@@ -111,8 +111,10 @@ static void x11_hide_mouse(Display *dpy, Window win)
    XFreeColors(dpy, colormap, &black.pixel, 1, 0);
 }
 
-void x11_show_mouse(Display *dpy, Window win, bool state)
+void x11_show_mouse(void *data, bool state)
 {
+   Display *dpy = g_x11_dpy;
+   Window   win = g_x11_win;
    if (state)
       XUndefineCursor(dpy, win);
    else
@@ -191,10 +193,8 @@ static void x11_set_window_pid(Display *dpy, Window win)
     if (gethostname(hostname, scret + 1) == -1)
         RARCH_WARN("Failed to get hostname.\n");
     else
-    {
         XChangeProperty(dpy, win, XA_WM_CLIENT_MACHINE, XA_STRING, 8,
             PropModeReplace, (unsigned char *)hostname, strlen(hostname));
-    }
     free(hostname);
 }
 
@@ -204,13 +204,35 @@ void x11_set_window_attr(Display *dpy, Window win)
    x11_set_window_pid(dpy, win);
 }
 
+#ifdef HAVE_XSCRNSAVER
+#include <X11/extensions/scrnsaver.h>
+static bool xss_screensaver_inhibit(Display *dpy, bool enable)
+{
+    int dummy, min, maj;
+    if (       !XScreenSaverQueryExtension(dpy, &dummy, &dummy)
+            || !XScreenSaverQueryVersion(dpy, &maj, &min)
+            || (maj < 1)
+            || (maj == 1 && min < 1))
+            return false;
+    XScreenSaverSuspend(dpy, enable);
+    XResetScreenSaver(dpy);
+    return true;
+}
+#else
+static bool xss_screensaver_inhibit(Display *dpy, bool enable)
+{
+    (void) dpy;
+    return false;
+}
+#endif
+
 static void xdg_screensaver_inhibit(Window wnd)
 {
    int  ret;
+   size_t _len;
    char cmd[64];
    char title[128];
 
-   cmd[0]   = '\0';
    title[0] = '\0';
 
    RARCH_LOG("[X11]: Suspending screensaver (X11, xdg-screensaver).\n");
@@ -228,7 +250,8 @@ static void xdg_screensaver_inhibit(Window wnd)
             8, PropModeReplace, (const unsigned char*) title, title_len);
    }
 
-   snprintf(cmd, sizeof(cmd), "xdg-screensaver suspend 0x%x", (int)wnd);
+   _len = strlcpy(cmd, "xdg-screensaver suspend 0x", sizeof(cmd));
+   snprintf(cmd + _len, sizeof(cmd) - _len, "%x", (int)wnd);
 
    if ((ret = system(cmd)) == -1)
    {
@@ -242,15 +265,25 @@ static void xdg_screensaver_inhibit(Window wnd)
    }
 }
 
-void x11_suspend_screensaver(Window wnd, bool enable)
+bool x11_suspend_screensaver(void *data, bool enable)
 {
+   Window wnd;
+   if (video_driver_display_type_get() != RARCH_DISPLAY_X11)
+      return false;
+   wnd = video_driver_window_get();
 #ifdef HAVE_DBUS
     if (dbus_suspend_screensaver(enable))
-       return;
+       return true;
 #endif
-    if (enable)
+    if (!xss_screensaver_inhibit(g_x11_dpy, enable) && enable)
+    {
        if (xdg_screensaver_available)
+       {
           xdg_screensaver_inhibit(wnd);
+          return xdg_screensaver_available;
+       }
+    }
+    return true;
 }
 
 #ifdef HAVE_XF86VM
@@ -284,14 +317,14 @@ static bool get_video_mode(
       Display *dpy, unsigned width, unsigned height,
       XF86VidModeModeInfo *mode, XF86VidModeModeInfo *x11_desktop_mode)
 {
-   int i, num_modes            = 0;
-   bool ret                    = false;
-   float refresh_mod           = 0.0f;
-   float minimum_fps_diff      = 0.0f;
-   XF86VidModeModeInfo **modes = NULL;
-   settings_t *settings        = config_get_ptr();
+   int i, num_modes                = 0;
+   bool ret                        = false;
+   float refresh_mod               = 0.0f;
+   float minimum_fps_diff          = 0.0f;
+   XF86VidModeModeInfo **modes     = NULL;
+   settings_t *settings            = config_get_ptr();
    unsigned black_frame_insertion  = settings->uints.video_black_frame_insertion;
-   float video_refresh_rate    = settings->floats.video_refresh_rate;
+   float video_refresh_rate        = settings->floats.video_refresh_rate;
 
    XF86VidModeGetAllModeLines(dpy, DefaultScreen(dpy), &num_modes, &modes);
 
@@ -418,7 +451,7 @@ static bool x11_create_input_context(Display *dpy,
    x11_init_keyboard_lut();
 
    g_x11_has_focus = true;
-   
+
    if (!(*xim = XOpenIM(dpy, NULL, NULL, NULL)))
    {
       RARCH_ERR("[X11]: Failed to open input method.\n");
@@ -466,7 +499,7 @@ bool x11_get_metrics(void *data,
          break;
       case DISPLAY_METRIC_DPI:
          dpy    = (Display*)XOpenDisplay(NULL);
-         *value = ((((float)DisplayWidth  (dpy, screen_no)) * 25.4) 
+         *value = ((((float)DisplayWidth  (dpy, screen_no)) * 25.4)
                /  (  (float)DisplayWidthMM(dpy, screen_no)));
          XCloseDisplay(dpy);
          break;
@@ -544,8 +577,8 @@ static void x11_handle_key_event(unsigned keycode, XEvent *event,
                (state & ShiftMask) || (state & LockMask));
    }
 
-   /* We can't feed uppercase letters to the keycode translator. 
-    * Seems like a bad idea to feed it keysyms anyway, so here 
+   /* We can't feed uppercase letters to the keycode translator.
+    * Seems like a bad idea to feed it keysyms anyway, so here
     * is a little hack...
     **/
    if (keysym >= XK_A && keysym <= XK_Z)
@@ -565,6 +598,8 @@ static void x11_handle_key_event(unsigned keycode, XEvent *event,
       mod |= RETROKMOD_ALT;
    if (state & Mod2Mask)
       mod |= RETROKMOD_NUMLOCK;
+   if (state & Mod3Mask)
+      mod |= RETROKMOD_SCROLLOCK;
    if (state & Mod4Mask)
       mod |= RETROKMOD_META;
 
@@ -639,6 +674,8 @@ bool x11_alive(void *data)
                case 5: /* Scroll down */
                case 6: /* Scroll wheel left */
                case 7: /* Scroll wheel right */
+               case 8: /* Mouse button 4 */
+               case 9: /* Mouse button 5 */
                   x_input_poll_wheel(&event.xbutton, true);
                   break;
             }
@@ -653,12 +690,19 @@ bool x11_alive(void *data)
             break;
 
          case ButtonRelease:
+            switch (event.xbutton.button)
+            {
+               case 8: /* Mouse button 4 - not handled as click */
+               case 9: /* Mouse button 5 - not handled as click */
+                  x_input_poll_wheel(&event.xbutton, true);
+                  break;
+            }
             break;
 
          case KeyRelease:
-            /*  When you receive a key release and the next event 
+            /*  When you receive a key release and the next event
              * is a key press of the same key combination,
-             * then it's auto-repeat and the key wasn't 
+             * then it's auto-repeat and the key wasn't
              * actually released. */
             if (XEventsQueued(g_x11_dpy, QueuedAfterReading))
             {
@@ -769,13 +813,13 @@ bool x11_connect(void)
 
 void x11_update_title(void *data)
 {
-   size_t len;
+   size_t _len;
    char title[128];
-   title[0] = '\0';
-   len      = video_driver_get_window_title(title, sizeof(title));
+   title[0]  = '\0';
+   _len      = video_driver_get_window_title(title, sizeof(title));
    if (title[0])
       XChangeProperty(g_x11_dpy, g_x11_win, XA_WM_NAME, XA_STRING,
-            8, PropModeReplace, (const unsigned char*)title, len);
+            8, PropModeReplace, (const unsigned char*)title, _len);
 }
 
 bool x11_input_ctx_new(bool true_full)
@@ -857,7 +901,7 @@ static bool x11_check_atom_supported(Display *dpy, Atom atom)
    if (!prop || type != XA_ATOM)
       return false;
 
-   for (i = 0; i < nitems; i++)
+   for (i = 0; i < (int)nitems; i++)
    {
       if (prop[i] == atom)
       {
