@@ -29,6 +29,7 @@
 #include <string.h>
 #include <math.h>
 
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <commdlg.h>
 
@@ -56,30 +57,12 @@ typedef struct gfx_ctx_w_vk_data
 } gfx_ctx_w_vk_data_t;
 
 /* TODO/FIXME - static globals */
-static gfx_ctx_vulkan_data_t win32_vk;
-static void             *dinput_vk        = NULL;
-static int              win32_vk_interval = 0;
+gfx_ctx_vulkan_data_t win32_vk;
+static void      *dinput_vk        = NULL;
+int              win32_vk_interval = 0;
 
-void create_vk_context(HWND hwnd, bool *quit)
-{
-   RECT rect;
-   HINSTANCE instance;
-   unsigned width  = 0;
-   unsigned height = 0;
-
-   GetClientRect(hwnd, &rect);
-
-   instance = GetModuleHandle(NULL);
-   width    = rect.right - rect.left;
-   height   = rect.bottom - rect.top;
-
-   if (!vulkan_surface_create(&win32_vk, VULKAN_WSI_WIN32,
-            &instance, &hwnd,
-            width, height, win32_vk_interval))
-      *quit = true;
-
-   g_win32_flags |= WIN32_CMN_FLAG_INITED;
-}
+/* FORWARD DECLARATIONS */
+void win32_get_video_size(void *data, unsigned *width, unsigned *height);
 
 static void gfx_ctx_w_vk_swap_interval(void *data, int interval)
 {
@@ -114,7 +97,7 @@ static void gfx_ctx_w_vk_check_window(void *data, bool *quit,
    if (     (win32_vk.flags & VK_DATA_FLAG_FULLSCREEN)
          && (g_win32_refresh_rate)
          && (g_win32_refresh_rate  != refresh_rate) 
-         && (abs(g_win32_refresh_rate - refresh_rate) > 0)
+         && (fabsf(g_win32_refresh_rate - refresh_rate) > 0.1f)
          && (g_win32_resize_width  == *width) 
          && (g_win32_resize_height == *height))
    {
@@ -154,47 +137,6 @@ static bool gfx_ctx_w_vk_set_resize(void *data,
    return false;
 }
 
-static void gfx_ctx_w_vk_update_title(void *data)
-{
-   char title[128];
-
-   title[0] = '\0';
-
-   video_driver_get_window_title(title, sizeof(title));
-
-   if (title[0])
-   {
-      const ui_window_t *window = ui_companion_driver_get_window_ptr();
-
-      if (window)
-         window->set_title(&main_window, title);
-   }
-}
-
-static void gfx_ctx_w_vk_get_video_size(void *data,
-      unsigned *width, unsigned *height)
-{
-   HWND         window     = win32_get_window();
-
-   if (!window)
-   {
-      RECT mon_rect;
-      MONITORINFOEX current_mon;
-      unsigned mon_id      = 0;
-      HMONITOR hm_to_use   = NULL;
-
-      win32_monitor_info(&current_mon, &hm_to_use, &mon_id);
-      mon_rect             = current_mon.rcMonitor;
-      *width               = mon_rect.right - mon_rect.left;
-      *height              = mon_rect.bottom - mon_rect.top;
-   }
-   else
-   {
-      *width               = g_win32_resize_width;
-      *height              = g_win32_resize_height;
-   }
-}
-
 static void gfx_ctx_w_vk_destroy(void *data)
 {
    HWND            window  = win32_get_window();
@@ -330,23 +272,29 @@ static void *gfx_ctx_w_vk_get_context_data(void *data) { return &win32_vk.contex
 
 static uint32_t gfx_ctx_w_vk_get_flags(void *data)
 {
-   uint32_t flags = 0;
+   uint32_t flags             = 0;
+   uint8_t present_mode_count = 16;
+   uint8_t i                  = 0;
+
+   /* Check for FIFO_RELAXED_KHR capability */
+   for (i = 0; i < present_mode_count; i++)
+   {
+      if (win32_vk.context.present_modes[i] == VK_PRESENT_MODE_FIFO_RELAXED_KHR)
+      {
+         BIT32_SET(flags, GFX_CTX_FLAGS_ADAPTIVE_VSYNC);
+         break;
+      }
+   }
+
 #if defined(HAVE_SLANG) && defined(HAVE_SPIRV_CROSS)
    BIT32_SET(flags, GFX_CTX_FLAGS_SHADERS_SLANG);
 #endif
+
    return flags;
 }
 
 static void gfx_ctx_w_vk_set_flags(void *data, uint32_t flags) { }
-
-static void gfx_ctx_w_vk_get_video_output_size(void *data,
-      unsigned *width, unsigned *height, char *desc, size_t desc_len)
-{
-   win32_get_video_output_size(width, height, desc, desc_len);
-}
-
 static void gfx_ctx_w_vk_get_video_output_prev(void *data) { }
-
 static void gfx_ctx_w_vk_get_video_output_next(void *data) { }
 
 const gfx_ctx_driver_t gfx_ctx_w_vk = {
@@ -356,18 +304,18 @@ const gfx_ctx_driver_t gfx_ctx_w_vk = {
    gfx_ctx_w_vk_bind_api,
    gfx_ctx_w_vk_swap_interval,
    gfx_ctx_w_vk_set_video_mode,
-   gfx_ctx_w_vk_get_video_size,
+   win32_get_video_size,
    win32_get_refresh_rate,
-   gfx_ctx_w_vk_get_video_output_size,
+   win32_get_video_output_size,
    gfx_ctx_w_vk_get_video_output_prev,
    gfx_ctx_w_vk_get_video_output_next,
    win32_get_metrics,
    NULL,
-   gfx_ctx_w_vk_update_title,
+   video_driver_update_title,
    gfx_ctx_w_vk_check_window,
    gfx_ctx_w_vk_set_resize,
    win32_has_focus,
-   win32_suppress_screensaver,
+   win32_suspend_screensaver,
    true,                            /* has_windowed */
    gfx_ctx_w_vk_swap_buffers,
    gfx_ctx_w_vk_input_driver,
