@@ -29,10 +29,26 @@
 #if defined(HAVE_LIBSHAKE)
 #include <shake.h>
 #include "../../configuration.h"
+#include "../../config.def.h"
 #endif
 
+/* RS-90 devices:
+ * - Analog input: No
+ * - Menu button:  No
+* RetroFW devices:
+ * - Analog input: No
+ * - Menu button:  Yes
+ * Miyoo devices:
+ * - Analog input: No
+ * - Menu button:  Yes
+ * All other OpenDingux devices:
+ * - Analog input: Yes
+ * - Menu button:  Yes
+ */
 #if !defined(RS90)
+#if !(defined(MIYOO) || defined(RETROFW))
 #define SDL_DINGUX_HAS_ANALOG      1
+#endif
 #define SDL_DINGUX_HAS_MENU_TOGGLE 1
 #endif
 
@@ -42,6 +58,71 @@
 
 #define SDL_DINGUX_JOYPAD_NAME "Dingux Gamepad"
 
+/* All digital inputs map to keyboard keys:
+ * - X:      SDLK_SPACE
+ * - A:      SDLK_LCTRL
+ * - B:      SDLK_LALT
+ * - Y:      SDLK_LSHIFT
+ * - L:      SDLK_TAB
+ * - R:      SDLK_BACKSPACE
+ * - L2:     SDLK_PAGEUP
+ * - R2:     SDLK_PAGEDOWN
+ * - Select: SDLK_ESCAPE
+ * - Start:  SDLK_RETURN
+ * - L3:     SDLK_KP_DIVIDE
+ * - R3:     SDLK_KP_PERIOD
+ * - Up:     SDLK_UP
+ * - Right:  SDLK_RIGHT
+ * - Down:   SDLK_DOWN
+ * - Left:   SDLK_LEFT
+ * - Menu:   SDLK_HOME
+ *
+ * Miyoo devices (Pocketgo, Powkiddy V90 & Q90)
+ * have the following alternate mappings:
+ * - X:      SDLK_LSHIFT
+ * - A:      SDLK_LALT
+ * - B:      SDLK_LCTRL
+ * - Y:      SDLK_SPACE
+ * - Menu:   SDLK_RCTRL
+ * - L3:     SDLK_RALT
+ * - R3:     SDLK_RSHIFT
+ */
+#if defined(MIYOO)
+#define SDL_DINGUX_SDLK_X      SDLK_LSHIFT
+#define SDL_DINGUX_SDLK_A      SDLK_LALT
+#define SDL_DINGUX_SDLK_B      SDLK_LCTRL
+#define SDL_DINGUX_SDLK_Y      SDLK_SPACE
+#else
+#define SDL_DINGUX_SDLK_X      SDLK_SPACE
+#define SDL_DINGUX_SDLK_A      SDLK_LCTRL
+#define SDL_DINGUX_SDLK_B      SDLK_LALT
+#define SDL_DINGUX_SDLK_Y      SDLK_LSHIFT
+#endif
+#define SDL_DINGUX_SDLK_L      SDLK_TAB
+#define SDL_DINGUX_SDLK_R      SDLK_BACKSPACE
+#define SDL_DINGUX_SDLK_L2     SDLK_PAGEUP
+#define SDL_DINGUX_SDLK_R2     SDLK_PAGEDOWN
+#define SDL_DINGUX_SDLK_SELECT SDLK_ESCAPE
+#define SDL_DINGUX_SDLK_START  SDLK_RETURN
+#if defined(MIYOO)
+#define SDL_DINGUX_SDLK_L3     SDLK_RALT
+#define SDL_DINGUX_SDLK_R3     SDLK_RSHIFT
+#else
+#define SDL_DINGUX_SDLK_L3     SDLK_KP_DIVIDE
+#define SDL_DINGUX_SDLK_R3     SDLK_KP_PERIOD
+#endif
+#define SDL_DINGUX_SDLK_UP     SDLK_UP
+#define SDL_DINGUX_SDLK_RIGHT  SDLK_RIGHT
+#define SDL_DINGUX_SDLK_DOWN   SDLK_DOWN
+#define SDL_DINGUX_SDLK_LEFT   SDLK_LEFT
+#if defined(RETROFW)
+#define SDL_DINGUX_SDLK_MENU   SDLK_END
+#elif defined(MIYOO)
+#define SDL_DINGUX_SDLK_MENU   SDLK_RCTRL
+#else
+#define SDL_DINGUX_SDLK_MENU   SDLK_HOME
+#endif
+
 #if defined(HAVE_LIBSHAKE)
 /* 5 ms period == 200 Hz
  * > Meissner's Corpuscle registers this
@@ -97,15 +178,11 @@ static dingux_joypad_t dingux_joypad;
 static bool sdl_dingux_rumble_init(dingux_joypad_rumble_t *rumble)
 {
    settings_t *settings = config_get_ptr();
-   unsigned rumble_gain = settings ? settings->uints.input_dingux_rumble_gain : 0;
+   unsigned rumble_gain = settings ? settings->uints.input_rumble_gain
+                                   : DEFAULT_RUMBLE_GAIN;
    bool weak_uploaded   = false;
    bool strong_uploaded = false;
 
-   /* If gain is zero, rumble is disabled
-    * > No need to initialise device */
-   if (rumble_gain == 0)
-      goto error;
-
    if (Shake_NumOfDevices() < 1)
       goto error;
 
@@ -117,8 +194,8 @@ static bool sdl_dingux_rumble_init(dingux_joypad_rumble_t *rumble)
 
    /* Check whether shake device has the required
     * feature set */
-   if (!Shake_QueryEffectSupport(rumble->device, SHAKE_EFFECT_PERIODIC) ||
-       !Shake_QueryWaveformSupport(rumble->device, SHAKE_PERIODIC_SINE))
+   if (   !Shake_QueryEffectSupport(rumble->device, SHAKE_EFFECT_PERIODIC)
+       || !Shake_QueryWaveformSupport(rumble->device, SHAKE_PERIODIC_SINE))
       goto error;
 
    /* In most cases it is recommended to use SHAKE_EFFECT_PERIODIC
@@ -160,8 +237,7 @@ static bool sdl_dingux_rumble_init(dingux_joypad_rumble_t *rumble)
    return true;
 
 error:
-   if (rumble_gain != 0)
-      RARCH_WARN("[libShake]: Input device does not support rumble effects.\n");
+   RARCH_WARN("[libShake]: Input device does not support rumble effects.\n");
 
    if (rumble->device)
    {
@@ -235,7 +311,7 @@ static bool sdl_dingux_joypad_set_rumble(unsigned pad,
 {
    dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
 
-   if (!joypad->rumble.device)
+   if ((pad != 0) || !joypad->rumble.device)
       return false;
 
    switch (effect)
@@ -254,6 +330,26 @@ static bool sdl_dingux_joypad_set_rumble(unsigned pad,
 
    return false;
 }
+
+static bool sdl_dingux_joypad_set_rumble_gain(unsigned pad, unsigned gain)
+{
+   dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
+
+   if ((pad != 0) || !joypad->rumble.device)
+      return false;
+
+   /* Gain is automatically capped by Shake_SetGain(),
+    * but do it explicitly here for clarity */
+   if (gain > 100)
+      gain = 100;
+
+   /* Set gain */
+   if (Shake_QueryGainSupport(joypad->rumble.device))
+      if (Shake_SetGain(joypad->rumble.device, (int)gain) == SHAKE_OK)
+         return true;
+
+   return false;
+}
 #endif
 
 static const char *sdl_dingux_joypad_name(unsigned port)
@@ -391,7 +487,7 @@ static bool sdl_dingux_joypad_query_pad(unsigned port)
    return (port == 0) && joypad->connected;
 }
 
-static int16_t sdl_dingux_joypad_button(unsigned port, uint16_t joykey)
+static int32_t sdl_dingux_joypad_button(unsigned port, uint16_t joykey)
 {
    dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
 
@@ -419,49 +515,48 @@ static void sdl_dingux_joypad_get_buttons(unsigned port, input_bits_t *state)
 static int16_t sdl_dingux_joypad_axis_state(unsigned port, uint32_t joyaxis)
 {
 #if defined(SDL_DINGUX_HAS_ANALOG)
-   dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
-   int val                 = 0;
-   int axis                = -1;
-   bool is_neg             = false;
-   bool is_pos             = false;
-
-   if (port != 0)
-      return 0;
-
-   if (AXIS_NEG_GET(joyaxis) < 4)
+   if (port == 0)
    {
-      axis   = AXIS_NEG_GET(joyaxis);
-      is_neg = true;
+      dingux_joypad_t *joypad = (dingux_joypad_t*)&dingux_joypad;
+      if (AXIS_NEG_GET(joyaxis) < 4)
+      {
+         int16_t val  = 0;
+         int16_t axis = AXIS_NEG_GET(joyaxis);
+         switch (axis)
+         {
+            case 0:
+            case 1:
+               val = joypad->analog_state[0][axis];
+               break;
+            case 2:
+            case 3:
+               val = joypad->analog_state[1][axis - 2];
+               break;
+         }
+         if (val < 0)
+            return val;
+      }
+      else if (AXIS_POS_GET(joyaxis) < 4)
+      {
+         int16_t val  = 0;
+         int16_t axis = AXIS_POS_GET(joyaxis);
+         switch (axis)
+         {
+            case 0:
+            case 1:
+               val = joypad->analog_state[0][axis];
+               break;
+            case 2:
+            case 3:
+               val = joypad->analog_state[1][axis - 2];
+               break;
+         }
+         if (val > 0)
+            return val;
+      }
    }
-   else if (AXIS_POS_GET(joyaxis) < 4)
-   {
-      axis   = AXIS_POS_GET(joyaxis);
-      is_pos = true;
-   }
-   else
-      return 0;
-
-   switch (axis)
-   {
-      case 0:
-      case 1:
-         val = joypad->analog_state[0][axis];
-         break;
-      case 2:
-      case 3:
-         val = joypad->analog_state[1][axis - 2];
-         break;
-   }
-
-   if (is_neg && val > 0)
-      return 0;
-   else if (is_pos && val < 0)
-      return 0;
-
-   return val;
-#else
-   return 0;
 #endif
+   return 0;
 }
 
 static int16_t sdl_dingux_joypad_axis(unsigned port, uint32_t joyaxis)
@@ -528,25 +623,7 @@ static void sdl_dingux_joypad_poll(void)
    }
 #endif
 
-   /* All digital inputs map to keyboard keys
-    * - X:      SDLK_SPACE
-    * - A:      SDLK_LCTRL
-    * - B:      SDLK_LALT
-    * - Y:      SDLK_LSHIFT
-    * - L:      SDLK_TAB
-    * - R:      SDLK_BACKSPACE
-    * - L2:     SDLK_PAGEUP
-    * - R2:     SDLK_PAGEDOWN
-    * - Select: SDLK_ESCAPE
-    * - Start:  SDLK_RETURN
-    * - L3:     SDLK_KP_DIVIDE
-    * - R3:     SDLK_KP_PERIOD
-    * - Up:     SDLK_UP
-    * - Right:  SDLK_RIGHT
-    * - Down:   SDLK_DOWN
-    * - Left:   SDLK_LEFT
-    * - Menu:   SDLK_HOME
-    */
+   /* All digital inputs map to keyboard keys */
 	while (SDL_PollEvent(&event))
 	{
 		switch (event.type)
@@ -554,56 +631,56 @@ static void sdl_dingux_joypad_poll(void)
 			case SDL_KEYDOWN:
             switch (event.key.keysym.sym)
             {
-               case SDLK_SPACE:
+               case SDL_DINGUX_SDLK_X:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_X);
                   break;
-               case SDLK_LCTRL:
+               case SDL_DINGUX_SDLK_A:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_A);
                   break;
-               case SDLK_LALT:
+               case SDL_DINGUX_SDLK_B:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_B);
                   break;
-               case SDLK_LSHIFT:
+               case SDL_DINGUX_SDLK_Y:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_Y);
                   break;
-               case SDLK_TAB:
+               case SDL_DINGUX_SDLK_L:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L);
                   break;
-               case SDLK_BACKSPACE:
+               case SDL_DINGUX_SDLK_R:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R);
                   break;
-               case SDLK_PAGEUP:
+               case SDL_DINGUX_SDLK_L2:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L2);
                   break;
-               case SDLK_PAGEDOWN:
+               case SDL_DINGUX_SDLK_R2:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R2);
                   break;
-               case SDLK_ESCAPE:
+               case SDL_DINGUX_SDLK_SELECT:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_SELECT);
                   break;
-               case SDLK_RETURN:
+               case SDL_DINGUX_SDLK_START:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_START);
                   break;
-               case SDLK_KP_DIVIDE:
+               case SDL_DINGUX_SDLK_L3:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L3);
                   break;
-               case SDLK_KP_PERIOD:
+               case SDL_DINGUX_SDLK_R3:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R3);
                   break;
-               case SDLK_UP:
+               case SDL_DINGUX_SDLK_UP:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_UP);
                   break;
-               case SDLK_RIGHT:
+               case SDL_DINGUX_SDLK_RIGHT:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_RIGHT);
                   break;
-               case SDLK_DOWN:
+               case SDL_DINGUX_SDLK_DOWN:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_DOWN);
                   break;
-               case SDLK_LEFT:
+               case SDL_DINGUX_SDLK_LEFT:
                   BIT16_SET(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_LEFT);
                   break;
 #if defined(SDL_DINGUX_HAS_MENU_TOGGLE)
-               case SDLK_HOME:
+               case SDL_DINGUX_SDLK_MENU:
                   BIT64_SET(lifecycle_state, RARCH_MENU_TOGGLE);
                   joypad->menu_toggle = true;
                   break;
@@ -615,52 +692,52 @@ static void sdl_dingux_joypad_poll(void)
 			case SDL_KEYUP:
             switch (event.key.keysym.sym)
             {
-               case SDLK_SPACE:
+               case SDL_DINGUX_SDLK_X:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_X);
                   break;
-               case SDLK_LCTRL:
+               case SDL_DINGUX_SDLK_A:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_A);
                   break;
-               case SDLK_LALT:
+               case SDL_DINGUX_SDLK_B:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_B);
                   break;
-               case SDLK_LSHIFT:
+               case SDL_DINGUX_SDLK_Y:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_Y);
                   break;
-               case SDLK_TAB:
+               case SDL_DINGUX_SDLK_L:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L);
                   break;
-               case SDLK_BACKSPACE:
+               case SDL_DINGUX_SDLK_R:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R);
                   break;
-               case SDLK_PAGEUP:
+               case SDL_DINGUX_SDLK_L2:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L2);
                   break;
-               case SDLK_PAGEDOWN:
+               case SDL_DINGUX_SDLK_R2:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R2);
                   break;
-               case SDLK_ESCAPE:
+               case SDL_DINGUX_SDLK_SELECT:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_SELECT);
                   break;
-               case SDLK_RETURN:
+               case SDL_DINGUX_SDLK_START:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_START);
                   break;
-               case SDLK_KP_DIVIDE:
+               case SDL_DINGUX_SDLK_L3:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_L3);
                   break;
-               case SDLK_KP_PERIOD:
+               case SDL_DINGUX_SDLK_R3:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_R3);
                   break;
-               case SDLK_UP:
+               case SDL_DINGUX_SDLK_UP:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_UP);
                   break;
-               case SDLK_RIGHT:
+               case SDL_DINGUX_SDLK_RIGHT:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_RIGHT);
                   break;
-               case SDLK_DOWN:
+               case SDL_DINGUX_SDLK_DOWN:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_DOWN);
                   break;
-               case SDLK_LEFT:
+               case SDL_DINGUX_SDLK_LEFT:
                   BIT16_CLEAR(joypad->pad_state, RETRO_DEVICE_ID_JOYPAD_LEFT);
                   break;
                default:
@@ -724,9 +801,13 @@ input_device_driver_t sdl_dingux_joypad = {
    sdl_dingux_joypad_poll,
 #if defined(HAVE_LIBSHAKE)
    sdl_dingux_joypad_set_rumble,
+   sdl_dingux_joypad_set_rumble_gain,
 #else
    NULL,
+   NULL,
 #endif
+   NULL, /* set_sensor_state */
+   NULL, /* get_sensor_input */
    sdl_dingux_joypad_name,
    "sdl_dingux",
 };
