@@ -32,6 +32,10 @@
 #include "../config.h"
 #endif /* HAVE_CONFIG_H */
 
+#if defined(_WIN32) && !defined(SOCKET)
+#include <winsock2.h>
+#endif
+
 #include "input_defines.h"
 #include "input_types.h"
 #ifdef HAVE_OVERLAY
@@ -40,8 +44,10 @@
 #include "input_osk.h"
 
 #include "../msg_hash.h"
+#ifdef HAVE_HID
 #include "include/hid_types.h"
 #include "include/hid_driver.h"
+#endif
 #include "include/gamepad.h"
 #include "../configuration.h"
 #include "../performance_counters.h"
@@ -78,7 +84,7 @@
 #define DEFAULT_MAX_PADS 4
 #elif defined(HAVE_ODROIDGO2)
 #define DEFAULT_MAX_PADS 8
-#elif defined(__linux__) || (defined(BSD) && !defined(__MACH__))
+#elif (defined(BSD) && !defined(__MACH__))
 #define DEFAULT_MAX_PADS 8
 #elif defined(__QNX__)
 #define DEFAULT_MAX_PADS 8
@@ -98,7 +104,39 @@
 #define MAPPER_SET_KEY(state, key) (state)->keys[(key) / 32] |= 1 << ((key) % 32)
 #define MAPPER_UNSET_KEY(state, key) (state)->keys[(key) / 32] &= ~(1 << ((key) % 32))
 
-#define INHERIT_JOYAXIS(binds) (((binds)[x_plus].joyaxis == (binds)[x_minus].joyaxis) || (  (binds)[y_plus].joyaxis == (binds)[y_minus].joyaxis))
+#define REPLAY_TOKEN_INVALID          '\0'
+#define REPLAY_TOKEN_REGULAR_FRAME    'f'
+#define REPLAY_TOKEN_CHECKPOINT_FRAME 'c'
+
+/**
+ * Takes as input analog key identifiers and converts them to corresponding
+ * bind IDs ident_minus and ident_plus.
+ *
+ * @param idx          Analog key index (eg RETRO_DEVICE_INDEX_ANALOG_LEFT)
+ * @param ident        Analog key identifier (eg RETRO_DEVICE_ID_ANALOG_X)
+ * @param ident_minus  Bind ID minus, will be set by function.
+ * @param ident_plus   Bind ID plus,  will be set by function.
+ */
+#define input_conv_analog_id_to_bind_id(idx, ident, ident_minus, ident_plus) \
+   switch ((idx << 1) | ident) \
+   { \
+      case (RETRO_DEVICE_INDEX_ANALOG_LEFT << 1) | RETRO_DEVICE_ID_ANALOG_X: \
+         ident_minus = RARCH_ANALOG_LEFT_X_MINUS; \
+         ident_plus  = RARCH_ANALOG_LEFT_X_PLUS; \
+         break; \
+      case (RETRO_DEVICE_INDEX_ANALOG_LEFT << 1) | RETRO_DEVICE_ID_ANALOG_Y: \
+         ident_minus = RARCH_ANALOG_LEFT_Y_MINUS; \
+         ident_plus  = RARCH_ANALOG_LEFT_Y_PLUS; \
+         break; \
+      case (RETRO_DEVICE_INDEX_ANALOG_RIGHT << 1) | RETRO_DEVICE_ID_ANALOG_X: \
+         ident_minus = RARCH_ANALOG_RIGHT_X_MINUS; \
+         ident_plus  = RARCH_ANALOG_RIGHT_X_PLUS; \
+         break; \
+      case (RETRO_DEVICE_INDEX_ANALOG_RIGHT << 1) | RETRO_DEVICE_ID_ANALOG_Y: \
+         ident_minus = RARCH_ANALOG_RIGHT_Y_MINUS; \
+         ident_plus  = RARCH_ANALOG_RIGHT_Y_PLUS; \
+         break; \
+   }
 
 RETRO_BEGIN_DECLS
 
@@ -108,6 +146,20 @@ enum rarch_movie_type
    RARCH_MOVIE_RECORD
 };
 
+enum input_driver_state_flags
+{
+   INP_FLAG_NONBLOCKING              = (1 << 0),
+   INP_FLAG_KB_LINEFEED_ENABLE       = (1 << 1),
+   INP_FLAG_KB_MAPPING_BLOCKED       = (1 << 2),
+   INP_FLAG_BLOCK_HOTKEY             = (1 << 3),
+   INP_FLAG_BLOCK_LIBRETRO_INPUT     = (1 << 4),
+   INP_FLAG_BLOCK_POINTER_INPUT      = (1 << 5),
+   INP_FLAG_GRAB_MOUSE_STATE         = (1 << 6),
+   INP_FLAG_REMAPPING_CACHE_ACTIVE   = (1 << 7),
+   INP_FLAG_DEFERRED_WAIT_KEYS       = (1 << 8),
+   INP_FLAG_WAIT_INPUT_RELEASE       = (1 << 9)
+};
+
 #ifdef HAVE_BSV_MOVIE
 enum bsv_flags
 {
@@ -129,32 +181,48 @@ struct bsv_state
 };
 
 /* These data are always little-endian. */
-struct bsv_key_data {
+struct bsv_key_data
+{
   uint8_t down;
-  uint16_t mod;
   uint8_t _padding;
+  uint16_t mod;
   uint32_t code;
   uint32_t character;
 };
-
 typedef struct bsv_key_data bsv_key_data_t;
 
+struct bsv_input_data
+{
+  uint8_t port;
+  uint8_t device;
+  uint8_t idx;
+  uint8_t _padding;
+  /* little-endian numbers */
+  uint16_t id;
+  int16_t value;
+};
+typedef struct bsv_input_data bsv_input_data_t;
+
 struct bsv_movie
 {
    intfstream_t *file;
    uint8_t *state;
+   int64_t identifier;
+   uint32_t version;
+   size_t min_file_pos;
+   size_t state_size;
+
    /* A ring buffer keeping track of positions
     * in the file for each frame. */
    size_t *frame_pos;
    size_t frame_mask;
-   size_t frame_ptr;
-   size_t min_file_pos;
-   size_t state_size;
-   int64_t identifier;
+   uint64_t frame_counter;
 
-   /* Staging variables for keyboard events */
+   /* Staging variables for events */
    uint8_t key_event_count;
-   bsv_key_data_t key_events[255];
+   uint16_t input_event_count;
+   bsv_key_data_t key_events[128];
+   bsv_input_data_t input_events[512];
 
    /* Rewind state */
    bool playback;
@@ -163,10 +231,6 @@ struct bsv_movie
 };
 
 typedef struct bsv_movie bsv_movie_t;
-
-#define REPLAY_TOKEN_INVALID          '\0'
-#define REPLAY_TOKEN_REGULAR_FRAME    'f'
-#define REPLAY_TOKEN_CHECKPOINT_FRAME 'c'
 #endif
 
 /**
@@ -193,9 +257,6 @@ struct input_keyboard_line
    bool enabled;
 };
 
-extern retro_keybind_set input_config_binds[MAX_USERS];
-extern retro_keybind_set input_autoconf_binds[MAX_USERS];
-
 struct rarch_joypad_info
 {
    const struct retro_keybind *auto_binds;
@@ -209,10 +270,9 @@ typedef struct
    uint16_t vid;
    uint16_t pid;
    char joypad_driver[32];
-   char name[256];
-   char display_name[256];
-   char config_path[PATH_MAX_LENGTH]; /* Path to the RetroArch config file */
-   char config_name[256]; /* Base name of the RetroArch config file */
+   char name[128];
+   char display_name[128];
+   char config_name[NAME_MAX_LENGTH]; /* Base name of the RetroArch config file */
    bool autoconfigured;
 } input_device_info_t;
 
@@ -228,15 +288,18 @@ struct remote_message
 struct input_remote
 {
 #if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
+#ifdef _WIN32
+   SOCKET net_fd[MAX_USERS];
+#else
    int net_fd[MAX_USERS];
+#endif
 #endif
    bool state[RARCH_BIND_LIST_END];
 };
 
-
 typedef struct
 {
-   char display_name[256];
+   char display_name[NAME_MAX_LENGTH];
 } input_mouse_info_t;
 
 typedef struct input_remote input_remote_t;
@@ -258,21 +321,20 @@ typedef struct input_list_element_t
    unsigned int state_size;
 } input_list_element;
 
-
 /**
  * Organizes the functions and data structures of each driver that are accessed
  * by other parts of the input code. The input_driver structs are the "interface"
  * between RetroArch and the input driver.
- * 
- * Every driver must establish an input_driver struct with pointers to its own 
+ *
+ * Every driver must establish an input_driver struct with pointers to its own
  * implementations of these functions, and each of those input_driver structs is
  * declared below.
  */
 struct input_driver
 {
-   /** 
+   /**
     * Initializes input driver.
-    * 
+    *
     * @param joypad_driver  Name of the joypad driver associated with the
     *                       input driver
     */
@@ -287,10 +349,10 @@ struct input_driver
     */
    void (*poll)(void *data);
 
-   /** 
+   /**
     * Queries state for a specified control on a specified input port. This
     * function pointer can be set to NULL if not supported by the input driver,
-    * for example if a joypad driver is responsible for quering state for a
+    * for example if a joypad driver is responsible for querying state for a
     * particular driver/platform.
     *
     * @param joypad_data      Input state struct, defined by the input driver
@@ -302,10 +364,10 @@ struct input_driver
     *                         with hardware device ID and autoconfig mapping.
     * @param retro_keybinds   Structure for control mappings for all libretro
     *                         input device abstractions
-    * @param keyboard_mapping_blocked 
+    * @param keyboard_mapping_blocked
     *                         If true, disregard custom keyboard mapping
     * @param port             Which RetroArch port is being polled
-    * @param device           Which libretro abstraction is being polled 
+    * @param device           Which libretro abstraction is being polled
     *                         (RETRO_DEVICE_ID_RETROPAD, RETRO_DEVICE_ID_MOUSE)
     * @param index            For controls with more than one axis or multiple
     *                         simultaneous inputs, such as an analog joystick
@@ -326,7 +388,7 @@ struct input_driver
 
    /**
     * Frees the input struct.
-    * 
+    *
     * @param data The input state struct.
     */
    void (*free)(void *data);
@@ -335,16 +397,16 @@ struct input_driver
     * Sets the state related for sensors, such as polling rate or to deactivate
     * the sensor entirely, etc. This function pointer may be set to NULL if
     * setting sensor values is not supported.
-    * 
-    * @param data    
+    *
+    * @param data
     * The input state struct
     * @param port
     * The port of the device
-    * @param effect  
+    * @param effect
     * Sensor action
-    * @param rate    
+    * @param rate
     * Sensor rate update
-    * 
+    *
     * @return true if the operation is successful.
    **/
    bool (*set_sensor_state)(void *data, unsigned port,
@@ -352,16 +414,16 @@ struct input_driver
 
    /**
     * Retrieves the sensor state associated with the provided port and ID. This
-    * function pointer may be set to NULL if retreiving sensor state is not
+    * function pointer may be set to NULL if retrieving sensor state is not
     * supported.
-    * 
-    * @param data  
+    *
+    * @param data
     * The input state struct
     * @param port
     * The port of the device
-    * @param id    
+    * @param id
     * Sensor ID
-    * 
+    *
     * @return The current state associated with the port and ID as a float
     **/
    float (*get_sensor_input)(void *data, unsigned port, unsigned id);
@@ -369,9 +431,9 @@ struct input_driver
    /**
     * The means for an input driver to indicate to RetroArch which libretro
     * input abstractions the driver supports.
-    * 
+    *
     * @param data  The input state struct.
-    * 
+    *
     * @return A unit64_t composed via bitwise operators.
     */
    uint64_t (*get_capabilities)(void *data);
@@ -385,7 +447,7 @@ struct input_driver
     * Grab or ungrab the mouse according to the value of `state`. This function
     * pointer can be set to NULL if the driver does not support grabbing the
     * mouse.
-    * 
+    *
     * @param data   The input state struct
     * @param state  True to grab the mouse, false to ungrab
     */
@@ -395,9 +457,9 @@ struct input_driver
     * Check to see if the input driver has claimed stdin, and therefore it is
     * not available for other input. This function pointercan be set to NULL if
     * the driver does not support claiming stdin.
-    * 
+    *
     * @param data  The input state struct
-    * 
+    *
     * @return True if the input driver has claimed stdin.
     */
    bool (*grab_stdin)(void *data);
@@ -422,6 +484,9 @@ struct rarch_joypad_driver
    void (*poll)(void);
    bool (*set_rumble)(unsigned, enum retro_rumble_effect, uint16_t);
    bool (*set_rumble_gain)(unsigned, unsigned);
+   bool (*set_sensor_state)(void *data, unsigned port,
+         enum retro_sensor_action action, unsigned rate);
+   float (*get_sensor_input)(void *data, unsigned port, unsigned id);
    const char *(*name)(unsigned);
 
    const char *ident;
@@ -429,7 +494,7 @@ struct rarch_joypad_driver
 
 /**
  * Callback for keypress events
- * 
+ *
  * @param userdata The user data that was passed through from the keyboard press callback.
  * @param code      keycode
  **/
@@ -441,21 +506,6 @@ struct input_keyboard_ctx_wait
    input_keyboard_press_t cb;
 };
 
-enum input_driver_state_flags
-{
-   INP_FLAG_NONBLOCKING              = (1 << 0),
-   INP_FLAG_KB_LINEFEED_ENABLE       = (1 << 1),
-   INP_FLAG_KB_MAPPING_BLOCKED       = (1 << 2),
-   INP_FLAG_BLOCK_HOTKEY             = (1 << 3),
-   INP_FLAG_BLOCK_LIBRETRO_INPUT     = (1 << 4),
-   INP_FLAG_BLOCK_POINTER_INPUT      = (1 << 5),
-   INP_FLAG_GRAB_MOUSE_STATE         = (1 << 6),
-   INP_FLAG_OLD_ANALOG_DPAD_MODE_SET = (1 << 7),
-   INP_FLAG_OLD_LIBRETRO_DEVICE_SET  = (1 << 8),
-   INP_FLAG_REMAPPING_CACHE_ACTIVE   = (1 << 9),
-   INP_FLAG_DEFERRED_WAIT_KEYS       = (1 << 10)
-};
-
 typedef struct
 {
    /**
@@ -484,18 +534,23 @@ typedef struct
 #endif
 #ifdef HAVE_BSV_MOVIE
    bsv_movie_t     *bsv_movie_state_handle;              /* ptr alignment */
+   bsv_movie_t     *bsv_movie_state_next_handle;         /* ptr alignment */
 #endif
 #ifdef HAVE_OVERLAY
    input_overlay_t *overlay_ptr;
+   input_overlay_t *overlay_cache_ptr;
    enum overlay_visibility *overlay_visibility;
    float overlay_eightway_dpad_slopes[2];
    float overlay_eightway_abxy_slopes[2];
+
+   /* touch pointer indexes from previous poll */
+   int old_touch_index_lut[OVERLAY_MAX_TOUCH];
 #endif
    uint16_t flags;
 #ifdef HAVE_NETWORKGAMEPAD
    input_remote_t *remote;
 #endif
-   char    *osk_grid[45];                                /* ptr alignment */ 
+   char    *osk_grid[45];                                /* ptr alignment */
 #if defined(HAVE_TRANSLATE)
 #if defined(HAVE_ACCESSIBILITY)
    int ai_gamepad_state[MAX_USERS];
@@ -505,10 +560,9 @@ typedef struct
    turbo_buttons_t turbo_btns; /* int32_t alignment */
 
    input_mapper_t mapper;          /* uint32_t alignment */
+   input_remap_cache_t remapping_cache;
    input_device_info_t input_device_info[MAX_INPUT_DEVICES]; /* unsigned alignment */
    input_mouse_info_t input_mouse_info[MAX_INPUT_DEVICES];
-   unsigned old_analog_dpad_mode[MAX_USERS];
-   unsigned old_libretro_device[MAX_USERS];
    unsigned osk_last_codepoint;
    unsigned osk_last_codepoint_len;
    unsigned input_hotkey_block_counter;
@@ -540,25 +594,25 @@ const char* config_get_input_driver_options(void);
 
 /**
  * Sets the rumble state.
- * 
- * @param port          
+ *
+ * @param port
  * User number.
  * @param joy_idx
  * TODO/FIXME ???
- * @param effect        
+ * @param effect
  * Rumble effect.
- * @param strength      
+ * @param strength
  * Strength of rumble effect.
  *
  * @return true if the rumble state has been successfully set
  **/
 bool input_driver_set_rumble(
-         unsigned port, unsigned joy_idx, 
+         unsigned port, unsigned joy_idx,
          enum retro_rumble_effect effect, uint16_t strength);
 /**
  * Sets the rumble gain.
  *
- * @param gain             
+ * @param gain
  * Rumble gain, 0-100 [%]
  * @param input_max_users
  * TODO/FIXME - ???
@@ -571,14 +625,14 @@ bool input_driver_set_rumble_gain(
 
 /**
  * Sets the sensor state.
- * 
+ *
  * @param port
  * User number.
  * @param sensors_enable
  * TODO/FIXME - ???
- * @param effect        
+ * @param effect
  * Sensor action
- * @param rate          
+ * @param rate
  * Sensor rate update
  *
  * @return true if the sensor state has been successfully set
@@ -588,13 +642,13 @@ bool input_driver_set_sensor(
          enum retro_sensor_action action, unsigned rate);
 
 /**
- * Retrieves the sensor state associated with the provided port and ID. 
- * 
+ * Retrieves the sensor state associated with the provided port and ID.
+ *
  * @param port
  * Port of the device
  * @param sensors_enable
  * TODO/FIXME - ???
- * @param id            
+ * @param id
  * Sensor ID
  *
  * @return The current state associated with the port and ID as a float
@@ -621,7 +675,7 @@ bool input_driver_ungrab_mouse(void);
 const char* config_get_joypad_driver_options(void);
 
 /**
- * Initialize a joypad driver of name ident. If ident points to NULL or a 
+ * Initialize a joypad driver of name ident. If ident points to NULL or a
  * zero-length string, equivalent to calling input_joypad_init_first().
  *
  * @param ident  identifier of driver to initialize.
@@ -633,42 +687,12 @@ const char* config_get_joypad_driver_options(void);
 const input_device_driver_t *input_joypad_init_driver(
       const char *ident, void *data);
 
-/**
- * Takes as input analog key identifiers and converts them to corresponding
- * bind IDs ident_minus and ident_plus.
- * 
- * @param idx          Analog key index (eg RETRO_DEVICE_INDEX_ANALOG_LEFT)
- * @param ident        Analog key identifier (eg RETRO_DEVICE_ID_ANALOG_X)
- * @param ident_minus  Bind ID minus, will be set by function.
- * @param ident_plus   Bind ID plus,  will be set by function.
- */
-#define input_conv_analog_id_to_bind_id(idx, ident, ident_minus, ident_plus) \
-   switch ((idx << 1) | ident) \
-   { \
-      case (RETRO_DEVICE_INDEX_ANALOG_LEFT << 1) | RETRO_DEVICE_ID_ANALOG_X: \
-         ident_minus = RARCH_ANALOG_LEFT_X_MINUS; \
-         ident_plus  = RARCH_ANALOG_LEFT_X_PLUS; \
-         break; \
-      case (RETRO_DEVICE_INDEX_ANALOG_LEFT << 1) | RETRO_DEVICE_ID_ANALOG_Y: \
-         ident_minus = RARCH_ANALOG_LEFT_Y_MINUS; \
-         ident_plus  = RARCH_ANALOG_LEFT_Y_PLUS; \
-         break; \
-      case (RETRO_DEVICE_INDEX_ANALOG_RIGHT << 1) | RETRO_DEVICE_ID_ANALOG_X: \
-         ident_minus = RARCH_ANALOG_RIGHT_X_MINUS; \
-         ident_plus  = RARCH_ANALOG_RIGHT_X_PLUS; \
-         break; \
-      case (RETRO_DEVICE_INDEX_ANALOG_RIGHT << 1) | RETRO_DEVICE_ID_ANALOG_Y: \
-         ident_minus = RARCH_ANALOG_RIGHT_Y_MINUS; \
-         ident_plus  = RARCH_ANALOG_RIGHT_Y_PLUS; \
-         break; \
-   }
-
 /**
  * Registers a newly connected pad with RetroArch.
- * 
- * @param port    
+ *
+ * @param port
  * Joystick number
- * @param driver  
+ * @param driver
  * Handle for joypad driver handling joystick's input
  **/
 void input_pad_connect(unsigned port, input_device_driver_t *driver);
@@ -676,14 +700,14 @@ void input_pad_connect(unsigned port, input_device_driver_t *driver);
 /**
  * Called by drivers when keyboard events are fired. Interfaces with the global
  * driver struct and libretro callbacks.
- * 
- * @param down       
+ *
+ * @param down
  * Was Keycode pressed down?
- * @param code       
+ * @param code
  * Keycode.
- * @param character  
+ * @param character
  * Character inputted.
- * @param mod        
+ * @param mod
  * TODO/FIXME/???
  **/
 void input_keyboard_event(bool down, unsigned code, uint32_t character,
@@ -693,11 +717,9 @@ input_driver_state_t *input_state_get_ptr(void);
 
 /*************************************/
 #ifdef HAVE_HID
-#include "include/hid_driver.h"
-
 /**
  * Get an enumerated list of all HID driver names
- * 
+ *
  * @return String listing of all HID driver names, separated by '|'.
  **/
 const char* config_get_hid_driver_options(void);
@@ -711,7 +733,7 @@ const hid_driver_t *input_hid_init_first(void);
 
 /**
  * Get a pointer to the HID driver data structure
- * 
+ *
  * @return Pointer to hid_data struct
  **/
 void *hid_driver_get_data(void);
@@ -728,7 +750,7 @@ void hid_driver_reset_data(void);
 
 /**
  * Set the name of the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
  */
@@ -736,59 +758,49 @@ void input_config_set_device_name(unsigned port, const char *name);
 
 /**
  * Set the formatted "display name" of the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
  */
 void input_config_set_device_display_name(unsigned port, const char *name);
 void input_config_set_mouse_display_name(unsigned port, const char *name);
 
-/**
- * Set the configuration path for the device in the specified port
- * 
- * @param port
- * The port of the device to be assigned to
- * @param path 
- * The path of the device config.
- */
-void input_config_set_device_config_path(unsigned port, const char *path);
-
 /**
  * Set the configuration name for the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
- * @param name 
+ * @param name
  * The name of the config to set.
  */
 void input_config_set_device_config_name(unsigned port, const char *name);
 
 /**
  * Set the joypad driver for the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
- * @param driver 
+ * @param driver
  * The driver to set the given port to.
  */
 void input_config_set_device_joypad_driver(unsigned port, const char *driver);
 
 /**
  * Set the vendor ID (vid) for the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
- * @param vid 
+ * @param vid
  * The VID to set the given device port to.
  */
 void input_config_set_device_vid(unsigned port, uint16_t vid);
 
 /**
  * Set the pad ID (pid) for the device in the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
- * @param pid 
+ * @param pid
  * The PID to set the given device port to.
  */
 void input_config_set_device_pid(unsigned port, uint16_t pid);
@@ -798,7 +810,7 @@ void input_config_set_device_pid(unsigned port, uint16_t pid);
  *
  * @param port
  * The port of the device to be assigned to
- * @param autoconfigured 
+ * @param autoconfigured
  * Whether or nor the device is configured automatically.
  */
 void input_config_set_device_autoconfigured(unsigned port, bool autoconfigured);
@@ -808,17 +820,17 @@ void input_config_set_device_autoconfigured(unsigned port, bool autoconfigured);
  *
  * @param port
  * The port of the device to be assigned to
- * @param name_index 
+ * @param name_index
  * The name index to set the device to use.
  */
 void input_config_set_device_name_index(unsigned port, unsigned name_index);
 
 /**
  * Sets the device type of the specified port
- * 
+ *
  * @param port
  * The port of the device to be assigned to
- * @param id 
+ * @param id
  * The device type (RETRO_DEVICE_JOYPAD, RETRO_DEVICE_MOUSE, etc)
  */
 void input_config_set_device(unsigned port, unsigned id);
@@ -826,7 +838,6 @@ void input_config_set_device(unsigned port, unsigned id);
 /* Clear input_device_info */
 void input_config_clear_device_name(unsigned port);
 void input_config_clear_device_display_name(unsigned port);
-void input_config_clear_device_config_path(unsigned port);
 void input_config_clear_device_config_name(unsigned port);
 void input_config_clear_device_joypad_driver(unsigned port);
 
@@ -840,13 +851,12 @@ unsigned input_config_get_device(unsigned port);
 const char *input_config_get_device_name(unsigned port);
 const char *input_config_get_device_display_name(unsigned port);
 const char *input_config_get_mouse_display_name(unsigned port);
-const char *input_config_get_device_config_path(unsigned port);
 const char *input_config_get_device_config_name(unsigned port);
 const char *input_config_get_device_joypad_driver(unsigned port);
 
 /**
  * Retrieves the vendor id (vid) of a connected controller
- * 
+ *
  * @param port
  * The port of the device
  *
@@ -856,7 +866,7 @@ uint16_t input_config_get_device_vid(unsigned port);
 
 /**
  * Retrieves the pad id (pad) of a connected controller
- * 
+ *
  * @param port
  * The port of the device
  *
@@ -876,7 +886,7 @@ bool input_config_get_device_autoconfigured(unsigned port);
 
 /**
  * Get the name index number for the device in this port
- * 
+ *
  * @param port
  * The port of the device
  *
@@ -893,7 +903,7 @@ unsigned input_config_get_device_name_index(unsigned port);
  * @deprecated input_config_get_device_name_ptr is required by linuxraw_joypad
  * and parport_joypad. These drivers should be refactored such that this
  * low-level access is not required.
- * 
+ *
  * @param port
  * The port of the device
  *
@@ -907,7 +917,7 @@ char *input_config_get_device_name_ptr(unsigned port);
  * @deprecated input_config_get_device_name_size is required by linuxraw_joypad
  * and parport_joypad. These drivers should be refactored such that this
  * low-level access is not required.
- * 
+ *
  * @param port
  * The port of the device
  *
@@ -915,6 +925,10 @@ char *input_config_get_device_name_ptr(unsigned port);
  */
 size_t input_config_get_device_name_size(unsigned port);
 
+unsigned input_driver_lightgun_id_convert(unsigned id);
+
+bool input_driver_pointer_is_offscreen(int16_t x, int16_t y);
+
 bool input_driver_button_combo(
       unsigned mode,
       retro_time_t current_time,
@@ -929,6 +943,9 @@ void input_keyboard_line_append(
       struct input_keyboard_line *keyboard_line,
       const char *word, size_t len);
 
+void input_keyboard_line_clear(input_driver_state_t *input_st);
+void input_keyboard_line_free(input_driver_state_t *input_st);
+
 /**
  * input_keyboard_start_line:
  * @userdata                 : Userdata.
@@ -957,18 +974,28 @@ void input_remote_free(input_remote_t *handle, unsigned max_users);
 
 void input_game_focus_free(void);
 
-void input_config_get_bind_string_joyaxis(
-      bool input_descriptor_label_show,
-      char *buf, const char *prefix,
-      const struct retro_keybind *bind, size_t size);
+/**
+ * Converts a retro_keybind to a human-readable string, optionally allowing a
+ * fallback auto_bind to be used as the source for the string.
+ *
+ * @param buf        A string which will be overwritten with the returned value
+ * @param bind       A binding to convert to a string
+ * @param auto_bind  A default binding which will be used after `bind`. Can be NULL.
+ * @param size       The maximum length that will be written to `buf`
+ */
+size_t input_config_get_bind_string(void *settings_data,
+      char *s, const struct retro_keybind *bind,
+      const struct retro_keybind *auto_bind, size_t len);
 
-void input_config_get_bind_string_joykey(
+size_t input_config_get_bind_string_joyaxis(
       bool input_descriptor_label_show,
-      char *buf, const char *prefix,
-      const struct retro_keybind *bind, size_t size);
+      char *s, const char *prefix,
+      const struct retro_keybind *bind, size_t len);
 
-int16_t input_state_internal(unsigned port, unsigned device,
-      unsigned idx, unsigned id);
+size_t input_config_get_bind_string_joykey(
+      bool input_descriptor_label_show,
+      char *s, const char *prefix,
+      const struct retro_keybind *bind, size_t len);
 
 bool input_key_pressed(int key, bool keyboard_pressed);
 
@@ -1003,16 +1030,21 @@ void input_driver_deinit_command(input_driver_state_t *input_st);
 #endif
 
 #ifdef HAVE_OVERLAY
-void input_overlay_deinit(void);
+void input_overlay_unload(void);
 
 void input_overlay_init(void);
+
+void input_overlay_check_mouse_cursor(void);
 #endif
 
 #ifdef HAVE_BSV_MOVIE
 void bsv_movie_frame_rewind(void);
 void bsv_movie_next_frame(input_driver_state_t *input_st);
+void bsv_movie_read_next_events(bsv_movie_t*handle);
 void bsv_movie_finish_rewind(input_driver_state_t *input_st);
 void bsv_movie_deinit(input_driver_state_t *input_st);
+void bsv_movie_deinit_full(input_driver_state_t *input_st);
+void bsv_movie_enqueue(input_driver_state_t *input_st, bsv_movie_t *state, enum bsv_flags flags);
 
 bool movie_start_playback(input_driver_state_t *input_st, char *path);
 bool movie_start_record(input_driver_state_t *input_st, char *path);
@@ -1057,9 +1089,9 @@ void input_driver_collect_system_input(input_driver_state_t *input_st,
  * @character                : Character inputted.
  * @mod                      : TODO/FIXME: ???
  *
- * Keyboard event utils. Called by drivers when keyboard events 
+ * Keyboard event utils. Called by drivers when keyboard events
  * are fired.
- * This interfaces with the global system driver struct 
+ * This interfaces with the global system driver struct
  * and libretro callbacks.
  **/
 void input_keyboard_event(bool down, unsigned code,
@@ -1097,6 +1129,7 @@ extern input_driver_t input_rwebinput;
 extern input_driver_t input_dos;
 extern input_driver_t input_winraw;
 extern input_driver_t input_wayland;
+extern input_driver_t input_test;
 
 extern input_device_driver_t dinput_joypad;
 extern input_device_driver_t linuxraw_joypad;
@@ -1120,6 +1153,7 @@ extern input_device_driver_t qnx_joypad;
 extern input_device_driver_t mfi_joypad;
 extern input_device_driver_t dos_joypad;
 extern input_device_driver_t rwebpad_joypad;
+extern input_device_driver_t test_joypad;
 
 #ifdef HAVE_HID
 extern hid_driver_t iohidmanager_hid;
@@ -1129,6 +1163,9 @@ extern hid_driver_t wiiusb_hid;
 extern hid_driver_t wiiu_hid;
 #endif /* HAVE_HID */
 
+extern retro_keybind_set input_config_binds[MAX_USERS];
+extern retro_keybind_set input_autoconf_binds[MAX_USERS];
+
 RETRO_END_DECLS
 
 #endif /* __INPUT_DRIVER__H */
