@@ -11,7 +11,7 @@
 
 #include <stdbool.h>
 
-static bool jb_has_debugger_attached(void);
+bool jit_available(void);
 bool jb_enable_ptrace_hack(void);
 
 #endif /* JITSupport_h */
